package zero1hd.rhythmbullet.audio.visualizer;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.utils.Disposable;

import zero1hd.rhythmbullet.audio.MusicController;

public class DoubleHorizontalVisualizer implements Disposable {
	private int width, height, barCount, barWidth, spaceBetweenBars;
	private int x, y;
	private ShapeRenderer shapeRenderer;
	private PCMSystem pcm;
	private float[] amplitudes;
	private int[] barHeights;
	private int binsPerBar;
	private float normalFactor = 20;
	private float barChangeRate = 4f;
	private int smoothRange = 1;
	/**
	 * 
	 * @param barCount amount of bars this visualizer should have.
	 * @param width the width of the visualizer.
	 * @param spacePercentage the percentage of a bar that should be space.
	 */
	public DoubleHorizontalVisualizer(int barCount, int width, int height, float spacePercentage, MusicController musicController, PCMSystem PCMSystem) {
		this.barCount = barCount;
		this.barWidth = width/barCount;
		this.spaceBetweenBars = MathUtils.round(barWidth * spacePercentage);
		this.barWidth -= spaceBetweenBars;
		pcm = PCMSystem;
		if (barWidth < 1) throw new IllegalArgumentException("The arguments you passed caused the bar width to be 0.");
		binsPerBar = (pcm.getFrequencyBins().length/barCount);
		this.width = width;
		this.height = height;
		amplitudes = new float[barCount];
		barHeights = new int[barCount];
		shapeRenderer = new ShapeRenderer();
	}
	
	public void act(float delta) {
		for (int bar = 0; bar < amplitudes.length; bar++) {
			float normalizedAmplitude = 0;
			for (int freq = bar*binsPerBar; freq < (bar*binsPerBar) + binsPerBar; freq++) {
				normalizedAmplitude += Math.abs(pcm.getFrequencyBins()[freq]);
			}
			amplitudes[bar] += normalizedAmplitude * normalFactor;
			amplitudes[bar] /= binsPerBar;
		}
		for (int bar = 0; bar < barHeights.length; bar++) {
			int smoothCount = 1;
			for (int range = 0; range < smoothRange; range++) {
				if (bar+range < amplitudes.length) {
					amplitudes[bar] += amplitudes[bar+range];
					smoothCount++;
				}
				if (bar-range > 0) {
					amplitudes[bar] += amplitudes[bar-range];
					smoothCount++;
				}
			}
			amplitudes[bar] /= smoothCount;
			
			int pixelsMoved = MathUtils.round(amplitudes[bar] - barHeights[bar]);
			pixelsMoved = MathUtils.floor(pixelsMoved*delta*barChangeRate);
			barHeights[bar] += pixelsMoved;
			
			if (barHeights[bar] < 0) barHeights[bar] = 0;
			if (barHeights[bar] > height) barHeights[bar] = height;
		}
	}
	
	public void draw(Batch batch, float parentAlpha) {
		batch.end();
		shapeRenderer.begin(ShapeType.Filled);
		shapeRenderer.setColor(1f, 1f, 1f, 1f);
		shapeRenderer.setProjectionMatrix(batch.getProjectionMatrix());
		shapeRenderer.setTransformMatrix(batch.getTransformMatrix());
		shapeRenderer.rect(x, y, width, 2);
		shapeRenderer.rect(x, y+height-2, width, 2);
		int beginX = x + spaceBetweenBars/2, beginY = y; 
		for (int bar = 0; bar < barCount; bar++) {
			shapeRenderer.rect(beginX + (spaceBetweenBars+barWidth)*bar, beginY+height, barWidth, barHeights[bar]);
			shapeRenderer.rect(beginX + (spaceBetweenBars+barWidth)*bar, beginY-barHeights[barHeights.length - 1 - bar], barWidth, barHeights[barHeights.length - 1 - bar]);
		}
		shapeRenderer.end();
		batch.begin();
	}
	
	public int getWidth() {
		return width;
	}
	
	public int getHeight() {
		return height;
	}
	
	public void setX(int x) {
		this.x = x;
	}
	
	public void setY(int y) {
		this.y = y;
	}
	
	public void setPosition(int x, int y) {
		this.x = x;
		this.y = y;
	}
	
	public int getX() {
		return x;
	}
	
	public int getY() {
		return y;
	}
	
	@Override
	public void dispose() {
		pcm.dispose();
	}
}
